/*-
 * #%L
 * Smile CDR - CDR
 * %%
 * Copyright (C) 2016 - 2024 Smile CDR, Inc.
 * %%
 * All rights reserved.
 * #L%
 */
package com.example.endpoint.hl7v2.inbound.term;

import ca.cdr.api.fhir.interceptor.CdrHook;
import ca.cdr.api.fhir.interceptor.CdrPointcut;
import ca.cdr.api.pub.hl7v2.model.Hl7v2ToFhirConversionResultJson;
import ca.cdr.api.pub.hl7v2.model.MappingMessage;
import ca.uhn.fhir.context.support.TranslateConceptResults;
import ca.uhn.fhir.interceptor.api.Interceptor;
import ca.uhn.fhir.jpa.api.model.TranslationRequest;
import ca.uhn.fhir.jpa.term.api.ITermConceptClientMappingSvc;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;

@Interceptor
public class TestPostConvertInterceptorCallingReverseTranslate {
	private static final Logger ourLog =
			LoggerFactory.getLogger(TestPostConvertInterceptorCallingReverseTranslate.class);

	@Autowired
	private ITermConceptClientMappingSvc myTermConceptMappingSvc;

	/**
	 * Demonstrating using an {@code ITermConceptClientMappingSvc} to have access to terminology translation
	 */
	@CdrHook(CdrPointcut.HL7V2IN_POST_HL7V2_TO_FHIR_MAPPING_PROCESSING)
	public void hl7v2PostProcess(Hl7v2ToFhirConversionResultJson theConversionResultJson) {
		try {
			ourLog.info("Interceptor HL7V2IN_POST_HL7V2_TO_FHIR_MAPPING_PROCESSING - started");

			// in a real use case we would translate some part of the theConversionResultJson message,
			// however we only care for translation service to be available in this example

			String targetSystem = "http://hl7.org/fhir/administrative-gender";
			String revertingCode = "F";

			ourLog.info("Reverse translating code '{}' to system: '{}'", revertingCode, targetSystem);
			TranslationRequest translationRequest = new TranslationRequest();
			translationRequest.getCodeableConcept().addCoding().setCode(revertingCode);
			translationRequest.setTargetSystem(targetSystem);
			translationRequest.setReverse(true);

			TranslateConceptResults result = myTermConceptMappingSvc.translate(translationRequest);

			String conversionResult = result.getResults().get(0).getCode();
			ourLog.info("Translation result: '{}'", conversionResult);

			if (result.getResult()) {
				String resultCode = result.getResults().get(0).getCode();
				ourLog.info("Reverse translation result: '{}'", resultCode);
				theConversionResultJson
						.getIssues()
						.add(new MappingMessage()
								.setLevel(MappingMessage.MessageLevel.INFO)
								.setMessage("Reverse translation result code: " + resultCode));
			} else {
				ourLog.info("Reverse translation failed");
				theConversionResultJson
						.getIssues()
						.add(new MappingMessage()
								.setLevel(MappingMessage.MessageLevel.ERROR)
								.setMessage("Reverse translation failed"));
			}

		} finally {
			ourLog.info("Interceptor HL7V2IN_POST_HL7V2_TO_FHIR_MAPPING_PROCESSING - ended");
		}
	}
}
