/*-
 * #%L
 * Smile CDR - CDR
 * %%
 * Copyright (C) 2016 - 2024 Smile CDR, Inc.
 * %%
 * All rights reserved.
 * #L%
 */
package com.example.endpoint.hl7v2.inbound.term;

import ca.cdr.api.fhir.interceptor.CdrHook;
import ca.cdr.api.fhir.interceptor.CdrPointcut;
import ca.cdr.api.pub.hl7v2.model.Hl7v2ToFhirConversionResultJson;
import ca.cdr.api.pub.hl7v2.model.MappingMessage;
import ca.uhn.fhir.context.support.TranslateConceptResults;
import ca.uhn.fhir.interceptor.api.Interceptor;
import ca.uhn.fhir.jpa.api.model.TranslationRequest;
import ca.uhn.fhir.jpa.term.api.ITermConceptClientMappingSvc;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;

@Interceptor
public class TestPreConvertInterceptorCallingTranslate {
	private static final Logger ourLog = LoggerFactory.getLogger(TestPreConvertInterceptorCallingTranslate.class);

	@Autowired
	private ITermConceptClientMappingSvc myTermConceptMappingSvc;

	/**
	 * Demonstrating using an {@code ITermConceptClientMappingSvc} to have access to terminology translation
	 */
	@CdrHook(CdrPointcut.HL7V2IN_PRE_HL7V2_TO_FHIR_MAPPING_PROCESSING)
	public Boolean hl7v2PreProcess(Hl7v2ToFhirConversionResultJson theConversionResultJson) {
		try {
			ourLog.info("Interceptor HL7V2IN_PRE_HL7V2_TO_FHIR_MAPPING_PROCESSING - started");

			// in a real use case we would translate some part of the theConversionResultJson message,
			// however we only care for translation service to be available in this example

			String fromSystem = "http://terminology.hl7.org/CodeSystem/v2-0001";
			String fromCode = "F";
			String toSystem = "http://hl7.org/fhir/administrative-gender";

			ourLog.info("Translating code: '{}' from system: '{}' to system: '{}'", fromCode, fromSystem, toSystem);
			TranslationRequest translationRequest =
					new TranslationRequest().addCode(fromSystem, fromCode).setTargetSystem(toSystem);

			TranslateConceptResults result = myTermConceptMappingSvc.translate(translationRequest);

			if (result.getResult()) {
				String resultCode = result.getResults().get(0).getCode();
				ourLog.info("Translation result: '{}'", resultCode);
				theConversionResultJson
						.getIssues()
						.add(new MappingMessage()
								.setLevel(MappingMessage.MessageLevel.INFO)
								.setMessage("Translation result code: " + resultCode));
			} else {
				ourLog.info("Translation failed");
				theConversionResultJson
						.getIssues()
						.add(new MappingMessage()
								.setLevel(MappingMessage.MessageLevel.ERROR)
								.setMessage("Translation failed"));
			}

		} finally {
			ourLog.info("Interceptor HL7V2IN_PRE_HL7V2_TO_FHIR_MAPPING_PROCESSING - ended");
		}
		return true; // continue processing
	}
}
