package com.example.endpoint.hl7v2.inbound;


import ca.cdr.api.pub.hl7v2.model.Hl7v2ToFhirConversionResultJson;
import ca.uhn.fhir.util.BundleUtil;
import ca.uhn.hl7v2.model.Message;
import org.hl7.fhir.instance.model.api.IBaseBundle;
import org.hl7.fhir.r4.model.Bundle;
import org.hl7.fhir.r4.model.Extension;
import org.hl7.fhir.r4.model.Patient;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;

import java.util.List;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertFalse;
import static org.junit.jupiter.api.Assertions.assertNotNull;

public class PostConvertInterceptorTest extends BaseHl7v2InterceptorTest {

	private PostConvertInterceptor myPostConvertInterceptor;

	@BeforeEach
	public void setup(){
		myPostConvertInterceptor = new PostConvertInterceptor();
	}

	@Test
	public void testExtensionAddedToPatientResources() throws Exception{
		Message message = createSampleAdtA01();
		Hl7v2ToFhirConversionResultJson conversionResult = new Hl7v2ToFhirConversionResultJson(message);

		Bundle bundle = new Bundle();

		Patient patient1 = createPatient("1", "John", "Smith");
		assertFalse(patient1.hasExtension(PostConvertInterceptor.EXTENSION_SYSTEM));
		bundle.addEntry().setResource(patient1);

		Patient patient2 = createPatient("2", "Jane", "Doe");
		assertFalse(patient2.hasExtension(PostConvertInterceptor.EXTENSION_SYSTEM));
		bundle.addEntry().setResource(patient2);

		conversionResult.addBundle(bundle);

		myPostConvertInterceptor.hl7v2ToFhirPostProcess(conversionResult);

		List<IBaseBundle> bundles = conversionResult.getBundles();
		assertEquals(1, bundles.size());

		List<Patient> patients = BundleUtil.toListOfResourcesOfType(myPostConvertInterceptor.getFhirContext(), bundles.get(0), Patient.class);
		assertEquals(2, patients.size());
		for (Patient patient : patients) {
			Extension extension = patient.getExtensionByUrl(PostConvertInterceptor.EXTENSION_SYSTEM);
			assertNotNull(extension);
			assertEquals(PostConvertInterceptor.EXTENSION_VALUE, extension.getValue().toString());
		}
	}

	private Patient createPatient(String theId, String theFirstName, String theLastName) {
		Patient patient = new Patient();
		patient.setId(theId);
		patient.getNameFirstRep().addGiven(theFirstName).setFamily(theLastName);
		return patient;
	}
}
